@@ -10,6 +10,7 @@
 import uuid
 import re
 import tempfile
+import json
 import warnings
 import time
 import dateutil.parser
@@ -21,7 +22,6 @@ from itertools import chain, cycle
 from mimetypes import guess_type
 from email.utils import parseaddr
 
-import simplejson as json
 from babel.dates import parse_date
 from nereid import (
     request, abort, render_template, login_required, url_for, redirect,
@@ -50,22 +50,14 @@ __metaclass__ = PoolMeta
 
 calendar.setfirstweekday(calendar.SUNDAY)
 PROGRESS_STATES = [
+    (None, ''),
     ('Backlog', 'Backlog'),
     ('Planning', 'Planning'),
     ('In Progress', 'In Progress'),
     ('Review', 'Review/QA'),
-    ('Done', 'Done'),
 ]
 
 
-def request_wants_json():
-    best = request.accept_mimetypes \
-        .best_match(['application/json', 'text/html'])
-    return best == 'application/json' and \
-        request.accept_mimetypes[best] > \
-        request.accept_mimetypes['text/html']
-
-
 class WebSite:
     """
     Website
@@ -99,7 +91,7 @@ class ProjectUsers(ModelSQL):
         super(ProjectUsers, cls).__setup__()
         cls._sql_constraints += [(
             'check_user',
-            'UNIQUE(project, "user")',
+            'UNIQUE(project, user)',
             'Users must be unique per project'
         )]
 
@@ -199,7 +191,7 @@ class ProjectInvitation(ModelSQL, ModelView):
 
         if request.method == 'POST':
             subject = '[%s] You have been re-invited to join the project' \
-                % self.project.name
+                % self.project.rec_name
             email_message = render_email(
                 text_template='project/emails/invite_2_project_text.html',
                 subject=subject, to=self.email,
@@ -278,24 +270,6 @@ class Project:
     """
     __name__ = 'project.work'
 
-    # This function field returns the project of a task even if the tree
-    # involves a much deeper subtask structure
-    project = fields.Function(fields.Many2One(
-        'project.work', 'Project',
-        domain=[
-            ('type', '=', 'project')
-        ], states={'invisible': Eval('type') != 'project'},
-    ), 'get_parent_project')
-
-    def get_parent_project(self, name):
-        """
-        Gets the parent of this project
-        """
-        # TODO: Make this function work even with highly nested tasks
-        if self.parent:
-            return self.parent.id
-        return self.id
-
     history = fields.One2Many(
         'project.work.history', 'project',
         'History', readonly=True
@@ -404,7 +378,7 @@ class Project:
 
         value = {
             'id': self.id,
-            'name': self.name,
+            'name': self.rec_name,
             'type': self.type,
             'parent': self.parent and self.parent.id or None,
             # Task specific
@@ -424,8 +398,6 @@ class Project:
             value['hours'] = self.hours
             value['effort'] = self.effort
             value['total_effort'] = self.total_effort
-            value['project'] = self.project and self.project.id
-            value['created_by'] = self.created_by and self.created_by._json()
         else:
             value['all_participants'] = [
                 participant._json() for participant in self.all_participants
@@ -439,11 +411,10 @@ class Project:
         rv = {
             'create_date': self.create_date.isoformat(),
             'id': self.id,
-            'displayName': self.name,
+            'displayName': self.rec_name,
             'type': self.type,
             'objectType': self.__name__,
         }
-        rv.update(self.serialize())
         if self.type == 'project':
             rv['url'] = url_for(
                 'project.work.render_project', project_id=self.id
@@ -495,23 +466,23 @@ class Project:
         return vals
 
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         '''
         Create a Project.
 
-        :param values: Values to create project.
+        :param vlist: List of dictionaries of values to create
         '''
-        if has_request_context():
-            values['created_by'] = request.nereid_user.id
-            if values['type'] == 'task':
-                values.setdefault('participants', [])
-                values['participants'].append(
-                    ('add', [request.nereid_user.id])
-                )
-        else:
-            # TODO: identify the nereid user through employee
-            pass
-        return super(Project, cls).create(values)
+        for values in vlist:
+            if has_request_context():
+                values['created_by'] = request.nereid_user.id
+                if values['type'] == 'task':
+                    values.setdefault('participants', []).append(
+                        ('add', [request.nereid_user.id])
+                    )
+            else:
+                # TODO: identify the nereid user through employee
+                pass
+        return super(Project, cls).create(vlist)
 
     def can_read(self, user):
         """
@@ -624,6 +595,8 @@ class Project:
         POST will create a new project
         """
         Activity = Pool().get('nereid.activity')
+        Work = Pool().get('timesheet.work')
+
         if not request.nereid_user.is_project_admin():
             flash(
                 "Sorry! You are not allowed to create new projects." +
@@ -632,16 +605,19 @@ class Project:
             return redirect(request.referrer)
 
         if request.method == 'POST':
-            project = cls.create({
-                'name': request.form['name'],
+            project, = cls.create([{
+                'work': Work.create([{
+                    'name': request.form['name'],
+                    'company': request.nereid_website.company.id,
+                }])[0].id,
                 'type': 'project',
-            })
-            Activity.create({
+            }])
+            Activity.create([{
                 'actor': request.nereid_user.id,
                 'object_': 'project.work, %d' % project.id,
                 'verb': 'created_project',
                 'project': project.id,
-            })
+            }])
             flash("Project successfully created.")
             return redirect(
                 url_for('project.work.render_project', project_id=project.id)
@@ -658,6 +634,7 @@ class Project:
         """
         NereidUser = Pool().get('nereid.user')
         Activity = Pool().get('nereid.activity')
+        Work = Pool().get('timesheet.work')
 
         project = self.get_project(self.id)
 
@@ -671,7 +648,10 @@ class Project:
                 name = request.form['name']
             data = {
                 'parent': self.id,
-                'name': name,
+                'work': Work.create([{
+                    'name': name,
+                    'company': request.nereid_website.company.id
+                }])[0].id,
                 'type': 'task',
                 'comment': request.form.get('description', None),
                 'tags': [('set', request.form.getlist('tags', int))]
@@ -688,14 +668,14 @@ class Project:
                 data['constraint_finish_time'] = datetime.strptime(
                     constraint_finish_time, '%m/%d/%Y')
 
-            task = self.create(data)
-            Activity.create({
+            task, = self.create([data])
+            Activity.create([{
                 'actor': request.nereid_user.id,
                 'object_': 'project.work, %d' % task.id,
                 'verb': 'created_task',
                 'target': 'project.work, %d' % project.id,
                 'project': project.id,
-            })
+            }])
 
             email_receivers = [p.email for p in self.all_participants]
             if request.form.get('assign_to', None):
@@ -710,7 +690,7 @@ class Project:
             task.send_mail(email_receivers)
             if request.is_xhr:
                 return jsonify(task.serialize())
-            flash("Task successfully added to project %s" % self.name)
+            flash("Task successfully added to project %s" % self.rec_name)
             return redirect(
                 url_for(
                     'project.work.render_task',
@@ -727,23 +707,27 @@ class Project:
         Edit the task
         """
         Activity = Pool().get('nereid.activity')
+        Work = Pool().get('timesheet.work')
+
         task = self.get_task(self.id)
 
-        self.write([task], {
+        Work.write([task.work], {
             'name': request.form.get('name'),
+        })
+        self.write([task], {
             'comment': request.form.get('comment')
         })
-        Activity.create({
+        Activity.create([{
             'actor': request.nereid_user.id,
             'object_': 'project.work, %d' % task.id,
             'verb': 'edited_task',
             'target': 'project.work, %d' % task.parent.id,
             'project': task.parent.id,
-        })
+        }])
         if request.is_xhr:
             return jsonify({
                 'success': True,
-                'name': self.name,
+                'name': self.rec_name,
                 'comment': self.comment,
             })
         return redirect(request.referrer)
@@ -754,7 +738,7 @@ class Project:
         :param receivers: Receivers of email.
         """
         subject = "[#%s %s] - %s" % (
-            self.id, self.parent.name, self.name
+            self.id, self.parent.rec_name, self.rec_name
         )
 
         if not receivers:
@@ -773,7 +757,7 @@ class Project:
             text_template='project/emails/project_text_content.jinja',
             html_template='project/emails/project_html_content.jinja',
             task=self,
-            updated_by=request.nereid_user.name
+            updated_by=request.nereid_user.display_name
         )
 
         #Send mail.
@@ -876,7 +860,7 @@ class Project:
         ], limit=1)
 
         subject = '[%s] You have been invited to join the project' \
-            % project.name
+            % project.rec_name
         if existing_user:
             # If participant already existed
             if existing_user[0] in project.participants:
@@ -895,20 +879,20 @@ class Project:
                     'participants': [('add', [existing_user[0].id])]
                 }
             )
-            Activity.create({
+            Activity.create([{
                 'actor': existing_user[0].id,
                 'object_': 'project.work, %d' % project.id,
                 'verb': 'joined_project',
                 'project': project.id,
-            })
+            }])
             flash_message = "%s has been invited to the project" \
                 % existing_user[0].display_name
 
         else:
-            new_invite = ProjectInvitation.create({
+            new_invite, = ProjectInvitation.create([{
                 'email': email,
                 'project': project.id,
-            })
+            }])
             email_message = render_email(
                 text_template='project/emails/invite_2_project_text.html',
                 subject=subject, to=email, from_email=CONFIG['smtp_from'],
@@ -964,13 +948,13 @@ class Project:
                         records_to_update_ids
                 ), {'participants': [('unlink', [participant_id])]}
             )
-            Activity.create({
+            Activity.create([{
                 'actor': request.nereid_user.id,
                 'object_': 'nereid.user, %d' % participant_id,
                 'target': 'project.work, %d' % self.id,
                 'verb': 'removed_participant',
                 'project': self.id,
-            })
+            }])
 
             return jsonify({
                 'success': True,
@@ -999,7 +983,7 @@ class Project:
             # This search is probably the suckiest search in the
             # history of mankind in terms of scalability and utility
             # TODO: Figure out something better
-            filter_domain.append(('name', 'ilike', '%%%s%%' % query))
+            filter_domain.append(('work.name', 'ilike', '%%%s%%' % query))
 
         tag = request.args.get('tag', None, int)
         if tag:
@@ -1058,7 +1042,7 @@ class Project:
             # This search is probably the suckiest search in the
             # history of mankind in terms of scalability and utility
             # TODO: Figure out something better
-            filter_domain.append(('name', 'ilike', '%%%s%%' % query))
+            filter_domain.append(('work.name', 'ilike', '%%%s%%' % query))
 
         tag = request.args.get('tag', None, int)
         if tag:
@@ -1088,7 +1072,7 @@ class Project:
             'project/global-task-list.jinja',
             active_type_name='render_task_list', counts=counts,
             state_filter=state, tasks_by_state=tasks_by_state,
-            states=PROGRESS_STATES[:-1]
+            states=PROGRESS_STATES
         )
 
     @classmethod
@@ -1100,21 +1084,18 @@ class Project:
         task = cls.get_task(task_id)
 
         comments = sorted(
-            task.history + task.timesheet_lines + task.attachments +
+            task.history + task.work.timesheet_lines + task.attachments +
             task.repo_commits, key=lambda x: x.create_date
         )
 
         hours = {}
-        for line in task.timesheet_lines:
+        for line in task.work.timesheet_lines:
             hours[line.employee] = hours.setdefault(line.employee, 0) + \
                 line.hours
 
         if request.is_xhr:
             response = cls.serialize(task)
-            with Transaction().set_context(task=task_id):
-                response['comments'] = [
-                    comment._json() for comment in comments
-                ]
+            response['comments'] = [comment._json() for comment in comments]
             return jsonify(response)
 
         return render_template(
@@ -1292,7 +1273,10 @@ class Project:
             'project/work-week.jinja', data_by_week=hours_by_week_employee,
             total_by_employee=total_by_employee
         )
-        return jsonify(day_totals=day_totals, lines=[], work_week=work_week)
+        return jsonify(
+            day_totals=day_totals, lines=[],
+            work_week=unicode(work_week)
+        )
 
     @classmethod
     @login_required
@@ -1543,7 +1527,7 @@ class Project:
             'project/tasks-by-employee.jinja',
             tasks_by_employee_by_state=tasks_by_employee_by_state,
             employees=employees,
-            states=PROGRESS_STATES[:-1],
+            states=PROGRESS_STATES,
         )
 
     @classmethod
@@ -1605,7 +1589,7 @@ class Project:
             def to_event(task, type="constraint"):
                 event = {
                     'id': task.id,
-                    'title': task.name,
+                    'title': task.rec_name,
                     'url': url_for(
                         'project.work.render_task',
                         project_id=task.parent.id, task_id=task.id),
@@ -1691,16 +1675,15 @@ class Project:
         attached_file = request.files["file"]
         resource = '%s,%d' % (cls.__name__, work.id)
 
-        filename = attached_file.filename
         if Attachment.search([
-            ('name', '=', filename),
+            ('name', '=', attached_file.filename),
             ('resource', '=', resource)
         ]):
-            # try to create a unique filename
-            filename, extension = filename.split('.', 1)
-            filename = '%s-%d.%s' % (
-                filename, time.time(), extension
+            flash(
+                'File already exists with same name, please choose another ' +
+                'file or rename this file to upload !!'
             )
+            return redirect(request.referrer)
 
         data = {
             'resource': resource,
@@ -1717,20 +1700,18 @@ class Project:
         else:
             data.update({
                 'data': attached_file.stream.read(),
-                'name': filename,
+                'name': attached_file.filename,
                 'type': 'data'
             })
 
-        attachment = Attachment.create(data)
+        Attachment.create([data])
 
-        if request.is_xhr or request_wants_json():
-            with Transaction().set_context(task=work.id):
-                return jsonify({
-                    'success': True,
-                    'data': attachment._json(),
-                })
+        if request.is_xhr:
+            return jsonify({
+                'success': True
+            })
 
-        flash("Attachment added to %s" % work.name)
+        flash("Attachment added to %s" % work.rec_name)
         return redirect(request.referrer)
 
     @classmethod
@@ -1753,7 +1734,7 @@ class Project:
             'comment': request.form['comment']
         }
 
-        updatable_attrs = ['progress_state']
+        updatable_attrs = ['state', 'progress_state']
         new_participant_ids = []
         current_participant_ids = [p.id for p in task.participants]
         post_attrs = [request.form.get(attr, None) for attr in updatable_attrs]
@@ -1765,11 +1746,6 @@ class Project:
                 if getattr(task, attr) != request.form.get(attr, None):
                     task_changes[attr] = request.form[attr]
 
-            if task_changes.get('progress_state') == 'Done':
-                task_changes['state'] = 'done'
-            else:
-                task_changes['state'] = 'opened'
-
             new_assignee_id = request.form.get('assigned_to', None, int)
             if not new_assignee_id is None:
                 if (new_assignee_id and
@@ -1793,17 +1769,17 @@ class Project:
                 # just create comment since nothing really changed since this
                 # update. This is to cover to cover cases where two users who
                 # havent refreshed the web page close the ticket
-                comment = History.create(history_data)
+                comment, = History.create([history_data])
         else:
             # Just comment, no update to task
-            comment = History.create(history_data)
-        Activity.create({
+            comment, = History.create([history_data])
+        Activity.create([{
             'actor': request.nereid_user.id,
             'object_': 'project.work.history, %d' % comment.id,
             'verb': 'updated_task',
             'target': 'project.work, %d' % task.id,
             'project': task.parent.id,
-        })
+        }])
 
         if request.nereid_user.id not in current_participant_ids:
             # Add the user to the participants if not already in the list
@@ -1822,18 +1798,18 @@ class Project:
 
         hours = request.form.get('hours', None, type=float)
         if hours and request.nereid_user.employee:
-            timesheet_line = TimesheetLine.create({
+            timesheet_line, = TimesheetLine.create([{
                 'employee': request.nereid_user.employee.id,
                 'hours': hours,
                 'work': task.work.id
-            })
-            Activity.create({
+            }])
+            Activity.create([{
                 'actor': request.nereid_user.id,
                 'object_': 'timesheet.line, %d' % timesheet_line.id,
                 'verb': 'reported_time',
                 'target': 'project.work, %d' % task.id,
                 'project': task.parent.id,
-            })
+            }])
 
         # Send the email since all thats required is done
         comment.send_mail()
@@ -1843,7 +1819,7 @@ class Project:
                 'project/comment.jinja', comment=comment)
             return jsonify({
                 'success': True,
-                'html': html,
+                'html': unicode(html),
                 'state': task.state,
                 'progress_state': task.progress_state,
                 'comment': comment._json(),
@@ -1865,16 +1841,16 @@ class Project:
         cls.write(
             [task], {'tags': [('add', [tag_id])]}
         )
-        Activity.create({
+        Activity.create([{
             'actor': request.nereid_user.id,
             'object_': 'project.work.tag, %d' % tag_id,
             'verb': 'added_tag_to_task',
             'target': 'project.work, %d' % task.id,
             'project': task.parent.id,
-        })
+        }])
 
         if request.method == 'POST':
-            flash('Tag added to task %s' % task.name)
+            flash('Tag added to task %s' % task.rec_name)
             return redirect(request.referrer)
 
         flash("Tag cannot be added")
@@ -1895,16 +1871,16 @@ class Project:
         cls.write(
             [task], {'tags': [('unlink', [tag_id])]}
         )
-        Activity.create({
+        Activity.create([{
             'actor': request.nereid_user.id,
             'object_': 'project.work, %d' % task.id,
             'verb': 'removed_tag_from_task',
             'target': 'project.work, %d' % task.parent.id,
             'project': task.parent.id,
-        })
+        }])
 
         if request.method == 'POST':
-            flash('Tag removed from task %s' % task.name)
+            flash('Tag removed from task %s' % task.rec_name)
             return redirect(request.referrer)
 
         flash("Tag cannot be removed")
@@ -1915,7 +1891,7 @@ class Project:
         """
         Update write to historize everytime an update is made
 
-        :param ids: ids of the projects
+        :param projects: List of active records of projects
         :param values: A dictionary
         """
         WorkHistory = Pool().get('project.work.history')
@@ -1942,13 +1918,13 @@ class Project:
         task = cls.get_task(task_id)
 
         with Transaction().set_user(0):
-            TimesheetLine.create({
+            TimesheetLine.create([{
                 'employee': request.nereid_user.employee.id,
                 'hours': request.form['hours'],
                 'work': task.work.id,
-            })
+            }])
 
-        flash("Time has been marked on task %s" % task.name)
+        flash("Time has been marked on task %s" % task.rec_name)
         return redirect(request.referrer)
 
     @classmethod
@@ -1967,7 +1943,7 @@ class Project:
         new_assignee = NereidUser(int(request.form['user']))
 
         if task.assigned_to == new_assignee:
-            flash("Task already assigned to %s" % new_assignee.name)
+            flash("Task already assigned to %s" % new_assignee.display_name)
             return redirect(request.referrer)
         if task.parent.can_write(new_assignee):
             cls.write([task], {
@@ -1975,18 +1951,18 @@ class Project:
                 'participants': [('add', [new_assignee.id])]
             })
             task.history[-1].send_mail()
-            Activity.create({
+            Activity.create([{
                 'actor': request.nereid_user.id,
                 'object_': 'project.work, %d' % task.id,
                 'verb': 'assigned_task_to',
                 'target': 'nereid.user, %d' % new_assignee.id,
                 'project': task.parent.id,
-            })
+            }])
             if request.is_xhr:
                 return jsonify({
                     'success': True,
                 })
-            flash("Task assigned to %s" % new_assignee.name)
+            flash("Task assigned to %s" % new_assignee.display_name)
             return redirect(request.referrer)
         flash("Only employees can be assigned to tasks.")
         return redirect(request.referrer)
@@ -2001,7 +1977,7 @@ class Project:
         task = cls.get_task(task_id)
 
         cls.write([task], {
-            'assigned_to': False
+            'assigned_to': None
         })
 
         if request.is_xhr:
@@ -2038,12 +2014,12 @@ class Project:
                 constraint_finish, '%m/%d/%Y')
 
         cls.write([task], data)
-        Activity.create({
+        Activity.create([{
             'actor': request.nereid_user.id,
             'object_': 'project.work, %d' % task.id,
             'verb': 'changed_date',
             'project': task.parent.id,
-        })
+        }])
 
         if request.is_xhr:
             return jsonify({
@@ -2107,9 +2083,7 @@ class Project:
         page = request.args.get('page', 1, int)
 
         domain = [('project', '=', self.id)]
-        activities = Pagination(
-            Activity, domain, page, request.args.get('limit', 20, type=int)
-        )
+        activities = Pagination(Activity, domain, page, 20)
         items = filter(
             None, map(lambda activity: activity.serialize(), activities)
         )
@@ -2181,18 +2155,18 @@ class Tag(ModelSQL, ModelView):
             return redirect(request.referrer)
 
         if request.method == 'POST':
-            tag = cls.create({
+            tag, = cls.create([{
                 'name': request.form['name'],
                 'color': request.form['color'],
                 'project': project.id
-            })
-            Activity.create({
+            }])
+            Activity.create([{
                 'actor': request.nereid_user.id,
                 'object_': 'project.work.tag, %d' % tag.id,
                 'verb': 'created_tag',
                 'target': 'project.work, %d' % project.id,
                 'project': project.id,
-            })
+            }])
 
             flash("Successfully created tag")
             return redirect(request.referrer)
@@ -2268,11 +2242,13 @@ class ProjectHistory(ModelSQL, ModelView):
     updated_by = fields.Many2One('nereid.user', 'Updated By')
     # States
     previous_state = fields.Selection([
+        (None, ''),
         ('opened', 'Opened'),
         ('done', 'Done'),
     ], 'Prev. State', select=True
     )
     new_state = fields.Selection([
+        (None, ''),
         ('opened', 'Opened'),
         ('done', 'Done'),
     ], 'New State', select=True
@@ -2326,7 +2302,6 @@ class ProjectHistory(ModelSQL, ModelView):
             "comment": self.comment,
             "new_state": self.new_state,
             "new_progress_state": self.new_progress_state,
-            "previous_progress_state": self.previous_progress_state,
             "new_assignee": (
                 self.new_assigned_to._json() if self.new_assigned_to
                     else None
@@ -2359,7 +2334,7 @@ class ProjectHistory(ModelSQL, ModelView):
                     # if an employee made the update
                     pass
                 data['project'] = project.id
-                return cls.create(data)
+                return cls.create([data])
 
     @login_required
     def update_comment(self, task_id):
@@ -2385,7 +2360,7 @@ class ProjectHistory(ModelSQL, ModelView):
             html = render_template('project/comment.jinja', comment=self)
             return jsonify({
                 'success': True,
-                'html': html,
+                'html': unicode(html),
                 'state': task.state,
             })
         return redirect(request.referrer)
@@ -2404,7 +2379,7 @@ class ProjectHistory(ModelSQL, ModelView):
 
         # Prepare the content of email.
         subject = "[#%s %s] - %s" % (
-            self.project.id, self.project.parent.name,
+            self.project.id, self.project.parent.rec_name,
             self.project.work.name,
         )
 
@@ -2498,7 +2473,7 @@ class ProjectWorkCommit(ModelSQL, ModelView):
                     ])
                     if commit_hook:
                         continue
-                    commit = cls.create({
+                    commit, = cls.create([{
                         'commit_timestamp': commit_timestamp,
                         'project': project,
                         'nereid_user': nereid_users[0].id,
@@ -2507,14 +2482,14 @@ class ProjectWorkCommit(ModelSQL, ModelView):
                         'commit_message': commit['message'],
                         'commit_url': commit['url'],
                         'commit_id': commit['id']
-                    })
-                    Activity.create({
+                    }])
+                    Activity.create([{
                         'actor': nereid_users[0].id,
                         'object_': 'project.work.commit, %d' % commit.id,
                         'verb': 'made_commit',
                         'target': 'project.work, %d' % project.id,
                         'project': project.parent.id,
-                    })
+                    }])
         return 'OK'
 
     def _json(self):
@@ -2527,7 +2502,7 @@ class ProjectWorkCommit(ModelSQL, ModelView):
             "displayName": self.commit_message,
             "repository": self.repository,
             "repository_url": self.repository_url,
-            "commit_timestamp": self.commit_timestamp.isoformat(),
+            "commit_timestamp": self.commit_timestamp,
             "commit_id": self.commit_id,
         }
 
@@ -2567,7 +2542,7 @@ class ProjectWorkCommit(ModelSQL, ModelView):
                     commit_timestamp = local_commit_time.astimezone(
                         dateutil.tz.tzutc()
                     )
-                    cls.create({
+                    cls.create([{
                         'commit_timestamp': commit_timestamp,
                         'project': project,
                         'nereid_user': nereid_users[0].id,
@@ -2584,7 +2559,7 @@ class ProjectWorkCommit(ModelSQL, ModelView):
                             commit['raw_node']
                         ),
                         'commit_id': commit['raw_node']
-                    })
+                    }])
         return 'OK'
 
 
@@ -2632,7 +2607,7 @@ def invitation_new_user_handler(nereid_user_id):
     nereid_user = NereidUser(nereid_user_id)
 
     subject = '[%s] %s Accepted the invitation to join the project' \
-        % (invitation.project.name, nereid_user.display_name)
+        % (invitation.project.rec_name, nereid_user.display_name)
 
     receivers = [
         p.email for p in invitation.project.company.project_admins if p.email
@@ -2652,12 +2627,12 @@ def invitation_new_user_handler(nereid_user_id):
             'participants': [('add', [nereid_user_id])]
         }
     )
-    Activity.create({
+    Activity.create([{
         'actor': nereid_user_id,
         'object_': 'project.work, %d' % invitation.project.id,
         'verb': 'joined_project',
         'project': invitation.project.id,
-    })
+    }])
 
 
 class TimesheetLine:
@@ -2713,18 +2688,9 @@ class Attachment:
     __name__ = "ir.attachment"
 
     def _json(self):
-        rv = {
+        return {
             'create_date': self.create_date.isoformat(),
             "objectType": self.__name__,
             "id": self.id,
             "updatedBy": self.uploaded_by._json(),
-            "displayName": self.name,
-            "description": self.description,
         }
-        if has_request_context():
-            rv['downloadUrl'] = url_for(
-                'project.work.download_file',
-                attachment_id=self.id,
-                task=Transaction().context.get('task'),
-            )
-        return rv
